import pyslang
from tables import param


class ParamVisitor:
    def __init__(self):
        self.params = []
        self.session = pyslang.ScriptSession()

    def visit(self, node):
        if node.kind == pyslang.SyntaxKind.ParameterDeclaration:     
            for declarator in node.declarators:
                self.params.append(param.ParamRow(
                    name=str(declarator.name).strip(),
                    dtype=str(node.type).strip() if str(node.type) != '' else None,
                    default_value=str(self.session.eval(str(declarator.initializer.expr))),
                    override_value=None,
                    scope=str(node.keyword).strip()
                ).series())
        elif node.kind == pyslang.SyntaxKind.TypeParameterDeclaration:
            print(f"Type {str(node.keyword).strip()}s not yet supported. (Identified: {str(node.declarators).strip()})")



import pandas as pd
from typing import Optional


class ParamRow():
    def __init__(
        self,
        name: str,
        dtype: str,
        scope: str,
        default_value: Optional[int] = None,
        override_value: Optional[int] = None,
    ):
        self.name = name
        self.dtype = dtype
        self.default_value = default_value
        self.override_value = override_value
        self.scope = scope

    def series(self) -> pd.Series:
        return pd.Series({
            "name": self.name,
            "dtype": self.dtype,
            "default_value": self.default_value,
            "override_value": self.override_value,
            "scope": self.scope,
        })

    def __hash__(self) -> int:
        return hash((self.name, self.dtype, self.scope, self.default_value, self.override_value))



def clean_port_string(port_str: str) -> str:
    """Clean and format port string"""
    parts = port_str.split("//")
    parts = [s.strip() for s in parts]
    parts = [s.split("\n") for s in parts]
    parts = [item for sublist in parts for item in sublist]

    return [s.strip() for s in parts][-1]


import pyslang
from utils import attr_utils

class SubmoduleInstanceVisitor:
    def __init__(self):
        self.submodule_instances = []

    def visit(self, node):
        if node.kind == pyslang.SyntaxKind.HierarchyInstantiation:
            attr_utils.print_attrs(node)
            # self.submodule_instances.append(node)
            # print("Submodule instance: ", node)
            print("--------------------------------")



from __future__ import annotations
import pandas as pd
from typing import Set, Optional


class PortRow():
    def __init__(
        self,
        name: str,
        dtype: str,
        direction: str,
        width: Optional[int] = 1,
        connected_to: Set[Port] = set(),
    ):
        self.name = name
        self.dtype = dtype
        self.width = width
        self.direction = direction
        self.connected_to = connected_to

    def __hash__(self) -> int:
        return hash((self.name, self.direction))
    
    def __eq__(self, other: Port) -> bool:
        if not isinstance(other, Port):
            return False
        return (self.name, self.direction) == (other.name, other.direction)

    def connect(self, other: Port):
        self.connected_to.add(other)
        other.connected_to.add(self)

    def series(self):
        return pd.Series({
            "name": self.name,
            "dtype": self.dtype,
            "direction": self.direction,
            "width": self.width,
            "connected_to": ','.join([p.name for p in self.connected_to]) if self.connected_to else None
        })


import pyslang
import pandas as pd
import pandasql
from ast_visitors import port_visitor, param_visitor, submodule_inst_visitor
from utils import attr_utils 

class Module:
    def __init__(self, file_paths: List[str]):
        self.file_paths = file_paths
        self.asts = [pyslang.SyntaxTree.fromFile(file_path) for file_path in file_paths]
        self.module = self.ast.root.members[0]

        # Module properties
        self.name = self.module.header.name.value        
        self.params = pd.DataFrame(columns=[
            "name", "dtype", "default_value", 
            "override_value", "scope", "typed_param"
        ])
        self.ports = pd.DataFrame(columns=[
            "name", "dtype", "width", "direction", "connected_to"
        ])
        self.tables = {
            "params": self.params,
            "ports": self.ports
        }

        # Build all tables
        self.build()

    def build(self) -> None:
        self._build_params()
        self._build_ports()
        self._build_submodule_instances()
        self._update_tables()

    def _build_params(self) -> None:
        root = self.module.header.parameters.declarations
        visitor = param_visitor.ParamVisitor()
        
        for decl in root: visitor.visit(decl)

        self.params = pd.DataFrame(visitor.params)

    def _build_ports(self) -> None:
        root = self.module.header.ports.ports
        visitor = port_visitor.PortVisitor()

        for port in root: visitor.visit(port)

        self.ports = pd.DataFrame(visitor.ports)

    def _build_submodule_instances(self) -> None:
        root = self.module.members
        visitor = submodule_inst_visitor.SubmoduleInstanceVisitor()

        for member in root: visitor.visit(member)


        pass

    def _update_tables(self) -> None:
        self.tables["params"] = self.params
        self.tables["ports"] = self.ports

    query = lambda self, q: pandasql.sqldf(q, self.tables)

    def print(self) -> None:
        print("Module: ", self.name)
        print("Parameters: \n", self.params)
        print("Ports: \n", self.ports)

    




import pyslang
from tables import port


class PortVisitor:
    def __init__(self):
        self.ports = []

    def visit(self, node):
        if node.kind == pyslang.SyntaxKind.ImplicitAnsiPort:
            self.ports.append(port.PortRow(
                name=str(node.declarator.name).strip(),
                dtype=str(node.header.dataType.keyword).strip() if str(node.header.dataType) != '' else None,
                width=str(node.header.dataType.dimensions).strip() if str(node.header.dataType.dimensions) != '' else '1',
                direction=str(node.header.direction).strip(),
                connected_to=None
            ).series())



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="svql",
    version="0.1.0",
    author="sathvik",
    author_email="sathvik@etched.com",
    description="A Python tool for parsing and analyzing SystemVerilog modules using SQL queries",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/etched-ai/svql",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)",
    ],
    python_requires=">=3.6",
    install_requires=[
        "pandas",
        "pandasql",
        "pyslang",
    ],
)
